#!/bin/bash

WITH_USR_PART=

FAKE_ROOT=/system

SYNC="/bin/sync"
REBOOT="/sbin/reboot"

INT_LOG_DIR=${FAKE_ROOT}/opt/data/update
INT_LOG_FILE=${INT_LOG_DIR}/fota_internal.log

CAT="/bin/cat"
MOUNT="/bin/mount"
UMOUNT="/bin/umount"
GREP="/bin/grep"
BLKID="/usr/sbin/blkid"

SYSTEM_DATA_MNT=opt
USER_MNT=opt/usr

#------------------------------------------------
#       get partition id
#------------------------------------------------
get_partition_id() {
    PART_ROOTFS=$("$BLKID" --match-token PARTLABEL=rootfs -o device || "$BLKID" --match-token LABEL=rootfs -o device)
    PART_SYSTEM_DATA=$("$BLKID" --match-token PARTLABEL=system-data -o device || "$BLKID" --match-token LABEL=system-data -o device)
    PART_USER=$("$BLKID" --match-token PARTLABEL=user -o device || "$BLKID" --match-token LABEL=user -o device)
}

#------------------------------------------------
#       check_user_partition
#------------------------------------------------
check_user_partition() {
    RET=$("$CAT" ${FAKE_ROOT}/etc/fstab | "$GREP" "user")
    if [ "z$RET" = "z" ]; then
        return 0
    fi
    return 1
}

#------------------------------------------------
#       mount_partitions
#------------------------------------------------
mount_partitions() {
    get_partition_id

    "$MOUNT" -t ext4 -o ro ${PART_ROOTFS} ${FAKE_ROOT}
    "$MOUNT" -t ext4 ${PART_SYSTEM_DATA} ${FAKE_ROOT}/${SYSTEM_DATA_MNT}

    check_user_partition
    WITH_USR_PART=$?
    if [ "$WITH_USR_PART" = "1" ]; then
        "$MOUNT" -t ext4 ${PART_USER} ${FAKE_ROOT}/${USER_MNT}
    fi

    "$MOUNT" -t proc none ${FAKE_ROOT}/proc
    "$MOUNT" -t sysfs none ${FAKE_ROOT}/sys
    "$MOUNT" -t devtmpfs devtmpfs ${FAKE_ROOT}/dev
    "$MOUNT" -t devpts devpts ${FAKE_ROOT}/dev/pts
    "$MOUNT" -t tmpfs tmpfs ${FAKE_ROOT}/tmp
}

#------------------------------------------------
#	do_reboot
#------------------------------------------------
do_reboot() {
	echo "Reboot"
	"$SYNC"
	"$REBOOT"
	while [ 1 ]
	do
		sleep 1
		echo "."
	done
}

#------------------------------------------------
#	do_rw_update
#------------------------------------------------
do_rw_update() {
	echo "Change into rw update mode" >> ${INT_LOG_FILE}

	# Pre-patches
	/bin/chroot ${FAKE_ROOT} /bin/sh -c /usr/share/upgrade/restore-passwd.sh

	# Go to system-update.target
	if [ $$ = 1 ]; then
		cd ${FAKE_ROOT}
		exec /bin/chroot . /sbin/init --unit=system-update.target $@
	fi
	do_reboot
}

#------------------------------------------------
#	Main Routine Start
#------------------------------------------------
mount_partitions

echo "rw update mode start ..." >> ${INT_LOG_FILE}
"$SYNC"

do_rw_update

